from merescocore.framework import Observable
from xml.sax.saxutils import escape as escapeXml
from subprocess import Popen, PIPE
from StringIO import StringIO
from tempfile import NamedTemporaryFile
from time import strftime, gmtime

from magic import open as open_magic, MAGIC_MIME
magicCookie = open_magic(MAGIC_MIME)
magicCookie.load()


RESPONSE_HEADER = """<fulltext xmlns="http://meresco.com/namespace/fulltext">
    <identifier>%(identifier)s</identifier>
    <datestamp>%(datestamp)s</datestamp>
"""
RESPONSE_ITEM = """
    <item>
        <url>%(url)s</url>
        <status>%(status)s</status>
        <errorCode>%(errorCode)s</errorCode>
        <data>%(data)s</data>
    </item>
"""
RESPONSE_FOOTER = """
</fulltext>
"""


class FullTextExtractor(Observable):

    def _isPdf(self, aFilename):
        return magicCookie.file(aFilename) == 'application/pdf'


    def addFullText(self, identifier=None, urls=None):
        recordData = ""
        recordData += RESPONSE_HEADER % {
            'identifier': escapeXml(identifier),
            'datestamp': self._datestamp(),
        }
        for url in urls:
            try:
                f = NamedTemporaryFile()
                for stuff in self.any.urlopen(url):
                    f.write(stuff)
                f.seek(0L)
                if not self._isPdf(f.name):
                    continue

                data = self._pdfToText(f.name)
                status = 'done'
                errorCode = ''
            except Exception, e:
                data = ''
                status = 'failed'
                errorCode = str(e)
            finally:
                f.close()
            recordData += RESPONSE_ITEM % {
                'url': escapeXml(url),
                'status': escapeXml(status),
                'errorCode': escapeXml(errorCode),
                'data': escapeXml(data),
                }
        recordData += RESPONSE_FOOTER
        self.any.sendReplace(identifier, recordSchema='fulltext', recordData=recordData)


    def _pdfToText(self, filename):
        outfile = NamedTemporaryFile()
        try:
            pipe = Popen(args=['pdftotext', filename, outfile.name], stdin=PIPE, stdout=PIPE, stderr=PIPE)
            out, errs = pipe.communicate()
            if errs:
                raise Exception(errs)
            result = ' '.join(outfile.read().strip().split())
        finally:
            outfile.close()

        return result

    def _datestamp(self):
        return strftime('%Y-%m-%dT%H:%M:%SZ', gmtime())
